/* Роботизированный погрузо-разгрузочный подъемный комплекс "RoboLift"
   
   Автор: Захаров Михаил
   2022 год 
*/

#include <Servo.h> // подключаем библиотеку для работы с сервоприводом
Servo servo1; // объявляем переменную Серво1 челюстного захвата
Servo servo2; // объявляем переменную Серво2 выдвижной платформы
Servo servo3; // объявляем переменную Серво3 наклона захвата
int angle1 = 93; // переменная для угла поворота Серво1 и её начальное значение
int angle2 = 12; // переменная для угла поворота Серво2 и её начальное значение
int angle3 = 75; // переменная для угла поворота Серво3 и её начальное значение

#include <Stepper.h> // подключаем библиотеку шагового мотора
Stepper LiftStepper(2048, 7, 5, 6, 4); // назначаем имя ШМ, кол-во шагов на 1об и пины

volatile boolean flag = false; // объявляем флаг
int led = 13 ; // назначаем пин светодиода

void setup ()  // объявляем начальные установки
{
  pinMode (led, OUTPUT);   // переводим пин светодиода в режим вывода
  servo1.attach(8); // привязываем Серво1 к пину 8
  servo1.write(angle1); // задаем начальное положение угла поворота Серво1
  servo2.attach(9); // привязываем Серво2 к пину 9
  servo2.write(angle2); // задаем начальное положение угла поворота Серво2
  servo3.attach(10); // привязываем Серво3 к пину 10
  servo3.write(angle3); // задаем начальное положение угла поворота Серво3
  LiftStepper.setSpeed(15); // устанавливаем номинальную скорость ШМ
  attachInterrupt(0, detector, RISING); // задаем прерывание датчика на приближение магнита
}

void detector ()  // объявляем выполнение функции прерывания
{
flag = true; // поднимаем флаг
}

void loop ()  // объявляем начало цикла работы
{
    if (flag) // условие, когда датчик сработал на появление магнитного поля
 { 
    digitalWrite (led, HIGH); // включается (горит) светодиод
    delay(2000); // пауза 2 секунды
    zagruzkaFunction(); // вызываем функцию загрузки подъемника
    delay(2000); // пауза 2 секунды
    liftupFuction(); // вызываем функцию подъема кабины
    delay(2000); // пауза 2 секунды    
    razgruzkaFunction(); // вызываем функцию разгрузки подъемника
    delay(2000); // пауза 2 секунды
    liftdownFuction(); // вызываем функцию спуска кабины
    flag = false;  // цикл выполнится один раз после появления магнитного поля
  }
  { // иначе, когда магнитного поля нет
    digitalWrite (led, LOW); // светодиод выключается (не горит)
  }
}

void zagruzkaFunction() //объявляем начало цикла загрузки подъемника
{
   while (angle1 > 60) // челюстной захват плавно раскрывается
   {
     angle1--;  // уменьшаем угол поворота Серво1 на 1 град.
     servo1.write(angle1);  // сообщаем Серво1 угол поворота
     delay(20);  // ждем 20 мс, пока Серво1 не сделает поворот
   }
   delay(2000); // пауза 2 секунды
   while (angle2 < 128)   // платформа плавно выдвигается
   {
     angle2++;  // увеличиваем угол поворота Серво2 на 1 град.
     servo2.write(angle2); // сообщаем Серво2 угол поворота
     delay(15); // ждем 15 мс, пока Серво2 не сделает поворот
    }
    delay(2000); // пауза 2 секунды
    while (angle1 < 93)   // челюстной захват плавно закрывается
    {
      angle1++;  // увеличиваем угол поворота серво1 на 1 град.
      servo1.write(angle1); // сообщаем серво1 угол поворота
      delay(20); // ждем 20 мс, пока Серво1 не сделает поворот
    }
    delay(2000); // пауза 2 секунды
    while (angle3 < 95)   // захват плавно приподнимается
    {
      angle3++;  // увеличиваем угол поворота Серво3 на 1 град.
      servo3.write(angle3); // сообщаем Серво3 угол поворота
      delay(20); // ждем 20 мс, пока Серво3 не сделает поворот
    }
    delay(2000); // пауза 2 секунды
    while (angle2 > 12) // платформа плавно задвигается
   {
     angle2--;  // уменьшаем угол поворота Серво2 на 1 град.
     servo2.write(angle2);  // сообщаем Серво2 угол поворота
     delay(15);  // ждем 15 мс, пока Серво2 не сделает поворот
   }
    return;
}

void razgruzkaFunction() //объявляем начало цикла разгрузки подъемника
{
     while (angle2 < 128)   // платформа плавно выдвигается
    {
      angle2++;  // увеличиваем угол поворота Серво2 на 1 град.
      servo2.write(angle2); // сообщаем Серво2 угол поворота
      delay(15); // ждем 15 мс, пока Серво2 не сделает поворот
    }
    delay(2000); // пауза 2 секунды
    while (angle3 > 75)   // захват плавно опускается
    {
      angle3--;  // уменьшаем угол поворота Серво3 на 1 град.
      servo3.write(angle3); // сообщаем Серво3 угол поворота
      delay(20); // ждем 20 мс, пока Серво3 не сделает поворот
    }
    delay(2000); // пауза 2 секунды
    while (angle1 > 60) // челюстной захват плавно раскрывается
   {
     angle1--;  // уменьшаем угол поворота Серво1 на 1 град.
     servo1.write(angle1);  // сообщаем Серво1 угол поворота
     delay(20);  // ждем 20 мс, пока Серво1 не сделает поворот
   }
    delay(2000); // пауза 2 секунды
    while (angle2 > 12) // платформа плавно задвигается
   {
     angle2--;  // уменьшаем угол поворота Серво2 на 1 град.
     servo2.write(angle2);  // сообщаем Серво2 угол поворота
     delay(15);  // ждем 15 мс, пока Серво2 не сделает поворот
   }
    delay(2000); // пауза 2 секунды
    while (angle1 < 93)   // челюстной захват плавно закрывается
    {
     angle1++;  // увеличиваем угол поворота серво1 на 1 град.
     servo1.write(angle1); // сообщаем серво1 угол поворота
     delay(20); // ждем 20 мс, пока Серво1 не сделает поворот
    }
    return;
}

void liftupFuction() // объявляем начало цикла подъема кабины
{
LiftStepper.step(5504); // выполняем подъем кабины
    return;
}

void liftdownFuction() // объявляем начало цикла спуска кабины
{
LiftStepper.step(-5504); // выполняем спуск кабины
    return;
}
